#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-2.1+

import os
import sys
import subprocess
import tempfile
from datetime import datetime, timedelta

def _build_certs():

    # expire in 7 days to avoid people using these in production
    dt_activation = datetime.utcnow().isoformat()
    dt_expiration = (datetime.utcnow()+timedelta(days=7)).isoformat()

    # certificate authority
    ca = 'ACME'
    ca_privkey = f'{ca}-CA.key'
    ca_certificate = f'{ca}-CA.pem'
    if not os.path.exists(ca_privkey):
        print('generating private key...')
        argv = ['certtool', '--generate-privkey',
                '--outfile', ca_privkey]
        rc = subprocess.run(argv, check=True)
        if rc.returncode != 0:
            return 1
    if not os.path.exists(ca_certificate):
        print('generating self-signed certificate...')

        # build config
        lines = []
        lines.append('organization = "ACME Corp."')
        lines.append('cn = "ACME CA"')
        lines.append('uri = "http://www.example.com/"')
        lines.append('email = "admin@example.com"')
        lines.append('crl_dist_points = "http://www.example.com/pki/"')
        lines.append('serial = 1')
        lines.append('crl_number = 1')
        lines.append('path_len = 1')
        lines.append('activation_date = "{}"'.format(dt_activation))
        lines.append('expiration_date = "{}"'.format(dt_expiration))
        lines.append('ca')
        lines.append('cert_signing_key')
        lines.append('crl_signing_key')
        lines.append('code_signing_key')
        cfg = tempfile.NamedTemporaryFile(mode='w',
                                          prefix='cert_',
                                          suffix=".cfg",
                                          dir=None,
                                          delete=True)
        cfg.write('\n'.join(lines))
        cfg.flush()
        argv = ['certtool',
                '--generate-self-signed',
                '--load-privkey', ca_privkey,
                '--template', cfg.name,
                '--outfile', ca_certificate]
        rc = subprocess.run(argv, check=True)
        if rc.returncode != 0:
            return 1

    # per-user key
    user = 'rhughes'
    user_privkey = f'{user}.key'
    user_certificate = f'{user}.pem'
    user_certificate_signed = f'{user}_signed.pem'
    user_request = f'{user}.csr'

    # build config
    lines = []
    lines.append('cn = "Richard Hughes"')
    lines.append('uri = "https://hughsie.com/"')
    lines.append('email = "richard@hughsie.com"')
    lines.append('activation_date = "{}"'.format(dt_activation))
    lines.append('expiration_date = "{}"'.format(dt_expiration))
    lines.append('signing_key')
    lines.append('code_signing_key')
    cfg = tempfile.NamedTemporaryFile(mode='w',
                                      prefix='cert_',
                                      suffix=".cfg",
                                      dir=None,
                                      delete=True)
    cfg.write('\n'.join(lines))
    cfg.flush()

    if not os.path.exists(user_privkey):
        print('generating user private key...')
        argv = ['certtool', '--generate-privkey', '--rsa',
                '--bits', '2048',
                '--outfile', user_privkey]
        rc = subprocess.run(argv, check=True)
        if rc.returncode != 0:
            return 1
    if not os.path.exists(user_certificate):
        print('generating self-signed certificate...')
        argv = ['certtool', '--generate-self-signed', '--rsa',
                '--bits', '2048',
                '--load-privkey', user_privkey,
                '--template', cfg.name,
                '--outfile', user_certificate]
        rc = subprocess.run(argv, check=True)
        if rc.returncode != 0:
            return 1

    if not os.path.exists(user_request):
        print('generating certificate...')
        argv = ['certtool', '--generate-request',
                '--load-privkey', user_privkey,
                '--template', cfg.name,
                '--outfile', user_request]
        rc = subprocess.run(argv, check=True)
        if rc.returncode != 0:
            return 1

    # sign the user
    if not os.path.exists(user_certificate_signed):
        print('generating CA-signed certificate...')
        argv = ['certtool', '--generate-certificate', '--rsa',
                '--bits', '2048',
                '--load-request', user_request,
                '--load-ca-certificate', ca_certificate,
                '--load-ca-privkey', ca_privkey,
                '--template', cfg.name,
                '--outfile', user_certificate_signed]
        rc = subprocess.run(argv, check=True)
        if rc.returncode != 0:
            return 1

    # success
    return 0

if __name__ == '__main__':
    sys.exit(_build_certs())



#!/usr/bin/python3
# pylint: disable=invalid-name,missing-docstring
#
# Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+

import sys
import xml.etree.ElementTree as ET

from pkg_resources import parse_version

XMLNS = '{http://www.gtk.org/introspection/core/1.0}'
XMLNS_C = '{http://www.gtk.org/introspection/c/1.0}'

def usage(return_code):
    """ print usage and exit with the supplied return code """
    if return_code == 0:
        out = sys.stdout
    else:
        out = sys.stderr
    out.write("usage: %s <NAME> <INPUT> <OUTPUT>\n" % sys.argv[0])
    sys.exit(return_code)

class LdVersionScript:
    """ Rasterize some text """

    def __init__(self, library_name):
        self.library_name = library_name
        self.releases = {}

    def _add_node(self, node):
        identifier = node.attrib[XMLNS_C + 'identifier']
        if 'version' not in node.attrib:
            print('No version for', identifier)
            sys.exit(1)
        version = node.attrib['version']
        if version not in self.releases:
            self.releases[version] = []
        release = self.releases[version]
        if identifier not in release:
            release.append(identifier)
        return version

    def _add_cls(self, cls):

        # add all class functions
        for node in cls.findall(XMLNS + 'function'):
            self._add_node(node)

        # choose the lowest version method for the _get_type symbol
        version_lowest = None
        if '{http://www.gtk.org/introspection/glib/1.0}get-type' not in cls.attrib:
            return
        type_name = cls.attrib['{http://www.gtk.org/introspection/glib/1.0}get-type']

        # add all class methods
        for node in cls.findall(XMLNS + 'method'):
            version_tmp = self._add_node(node)
            if version_tmp:
                if not version_lowest or parse_version(version_tmp) < parse_version(version_lowest):
                    version_lowest = version_tmp

        # add the constructor
        for node in cls.findall(XMLNS + 'constructor'):
            version_tmp = self._add_node(node)
            if version_tmp:
                if not version_lowest or parse_version(version_tmp) < parse_version(version_lowest):
                    version_lowest = version_tmp

        # finally add the get_type symbol
        if version_lowest:
            self.releases[version_lowest].append(type_name)

    def import_gir(self, filename):
        tree = ET.parse(filename)
        root = tree.getroot()
        for ns in root.findall(XMLNS + 'namespace'):
            for node in ns.findall(XMLNS + 'function'):
                self._add_node(node)
            for cls in ns.findall(XMLNS + 'record'):
                self._add_cls(cls)
            for cls in ns.findall(XMLNS + 'class'):
                self._add_cls(cls)

    def render(self):

        # get a sorted list of all the versions
        versions = []
        for version in self.releases:
            versions.append(version)

        # output the version data to a file
        verout = '# generated automatically, do not edit!\n'
        oldversion = None
        for version in sorted(versions, key=parse_version):
            symbols = sorted(self.releases[version])
            verout += '\n%s_%s {\n' % (self.library_name, version)
            verout += '  global:\n'
            for symbol in symbols:
                verout += '    %s;\n' % symbol
            verout += '  local: *;\n'
            if oldversion:
                verout += '} %s_%s;\n' % (self.library_name, oldversion)
            else:
                verout += '};\n'
            oldversion = version
        return verout

if __name__ == '__main__':
    if {'-?', '--help', '--usage'}.intersection(set(sys.argv)):
        usage(0)
    if len(sys.argv) != 4:
        usage(1)

    ld = LdVersionScript(library_name=sys.argv[1])
    ld.import_gir(sys.argv[2])
    open(sys.argv[3], 'w').write(ld.render())
